summaryrefslogtreecommitdiff
path: root/src/SMAPI/Framework
diff options
context:
space:
mode:
authorKhloe Leclair <xivkhloeleclair@gmail.com>2022-09-26 15:18:36 -0400
committerKhloe Leclair <xivkhloeleclair@gmail.com>2022-09-26 15:18:36 -0400
commite8da8fff5163eacd6ae7870eaa8c7dbc8285e3e7 (patch)
tree1543697e82d2ccb74d293688e122186edac394ee /src/SMAPI/Framework
parente7d29a2f7dabde75fb1ad76af1975c9194b1b8bd (diff)
downloadSMAPI-e8da8fff5163eacd6ae7870eaa8c7dbc8285e3e7.tar.gz
SMAPI-e8da8fff5163eacd6ae7870eaa8c7dbc8285e3e7.tar.bz2
SMAPI-e8da8fff5163eacd6ae7870eaa8c7dbc8285e3e7.zip
Initial work on a way for mods to return specific API instances to specific mods.
Diffstat (limited to 'src/SMAPI/Framework')
-rw-r--r--src/SMAPI/Framework/ModHelpers/ModRegistryHelper.cs53
1 files changed, 47 insertions, 6 deletions
diff --git a/src/SMAPI/Framework/ModHelpers/ModRegistryHelper.cs b/src/SMAPI/Framework/ModHelpers/ModRegistryHelper.cs
index 348ba225..9ad3e3ae 100644
--- a/src/SMAPI/Framework/ModHelpers/ModRegistryHelper.cs
+++ b/src/SMAPI/Framework/ModHelpers/ModRegistryHelper.cs
@@ -1,5 +1,7 @@
+using System;
using System.Collections.Generic;
using StardewModdingAPI.Framework.Reflection;
+using StardewModdingAPI.Internal;
namespace StardewModdingAPI.Framework.ModHelpers
{
@@ -15,8 +17,8 @@ namespace StardewModdingAPI.Framework.ModHelpers
/// <summary>Encapsulates monitoring and logging for the mod.</summary>
private readonly IMonitor Monitor;
- /// <summary>The mod IDs for APIs accessed by this instanced.</summary>
- private readonly HashSet<string> AccessedModApis = new();
+ /// <summary>The APIs accessed by this instance.</summary>
+ private readonly Dictionary<string, object?> AccessedModApis = new();
/// <summary>Generates proxy classes to access mod APIs through an arbitrary interface.</summary>
private readonly IInterfaceProxyFactory ProxyFactory;
@@ -66,11 +68,50 @@ namespace StardewModdingAPI.Framework.ModHelpers
return null;
}
- // get raw API
+ // get our cached API if one is available
IModMetadata? mod = this.Registry.Get(uniqueID);
- if (mod?.Api != null && this.AccessedModApis.Add(mod.Manifest.UniqueID))
- this.Monitor.Log($"Accessed mod-provided API for {mod.DisplayName}.");
- return mod?.Api;
+ if (mod == null)
+ return null;
+
+ if (this.AccessedModApis.ContainsKey(mod.Manifest.UniqueID))
+ {
+ return this.AccessedModApis[mod.Manifest.UniqueID];
+ }
+
+ object? api;
+
+ // safely request a specific API instance
+ try
+ {
+ api = mod.Mod?.GetApi(this.Mod.Manifest);
+ if (api != null && !api.GetType().IsPublic)
+ {
+ api = null;
+ this.Monitor.Log($"{mod.DisplayName} provided a specific API instance with a non-public type. This isn't currently supported, so the specific API won't be available to the requesting mod.", LogLevel.Warn);
+ }
+
+ if (api != null)
+ this.Monitor.Log($"Accessed specific mod-provided API ({api.GetType().FullName}) for {mod.DisplayName}.");
+ }
+ catch (Exception ex)
+ {
+ this.Monitor.Log($"Failed loading specific mod-provided API for {mod.DisplayName}. Integrations with other mods may not work. Error: {ex.GetLogSummary()}", LogLevel.Error);
+ api = null;
+ }
+
+ // fall back to the generic API instance
+ if (api == null)
+ {
+ api = mod.Api;
+ if (api != null)
+ {
+ this.Monitor.Log($"Accessed mod-provided API for {mod.DisplayName}.");
+ }
+ }
+
+ // cache the API instance and return it
+ this.AccessedModApis[mod.Manifest.UniqueID] = api;
+ return api;
}
/// <inheritdoc />